//this file is part of notepad++
//Copyright (C)2003 Don HO ( donho@altern.org )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef PLUGINSMANAGER_H
#define PLUGINSMANAGER_H

#include "resource.h"
#include "Parameters.h"
#include "PluginInterface.h"


struct PluginInfo {
	PluginInfo() :_hLib(NULL), _pluginMenu(NULL), _pFuncSetInfo(NULL),\
		_pFuncGetFuncsArray(NULL), _pFuncGetName(NULL), _funcItems(NULL),\
		_nbFuncItem(0){};
	~PluginInfo(){
		if (_pluginMenu)
			::DestroyMenu(_pluginMenu);

		if (_hLib)
			::FreeLibrary(_hLib);
	};

	HINSTANCE _hLib;
	HMENU _pluginMenu;

	PFUNCSETINFO _pFuncSetInfo;
	PFUNCGETNAME _pFuncGetName;
	PBENOTIFIED	_pBeNotified;
	PFUNCGETFUNCSARRAY _pFuncGetFuncsArray;
	
	FuncItem *_funcItems;
	int _nbFuncItem;
};

class PluginsManager {
public:
	PluginsManager() : _hPluginsMenu(NULL) {};
	~PluginsManager() {
		
		for (size_t i = 0 ; i < _pluginInfos.size() ; i++)
			delete _pluginInfos[i];

		if (_hPluginsMenu)
			DestroyMenu(_hPluginsMenu);
	};
	void init(const NppData & nppData) {
		_nppData = nppData;
	};
	bool loadPlugins();
	
	void runPluginCommand(size_t i) {
		if (_pluginsCommands[i])
			_pluginsCommands[i]();
	};

	void setMenu(HMENU hMenu) {
		if (hasPlugins())
		{
			_hPluginsMenu = ::CreateMenu();
			::InsertMenu(hMenu, 10, MF_BYPOSITION | MF_POPUP, (UINT_PTR)_hPluginsMenu, "Plugins");

			for (size_t i = 0 ; i < _pluginInfos.size() ; i++)
			{
				::InsertMenu(_hPluginsMenu, i, MF_BYPOSITION | MF_POPUP, (UINT_PTR)_pluginInfos[i]->_pluginMenu, _pluginInfos[i]->_pFuncGetName());

				for (int j = 0 ; j < _pluginInfos[i]->_nbFuncItem ; j++)
				{
					_pluginsCommands.push_back(_pluginInfos[i]->_funcItems[j]._pFunc);
					int cmdID = ID_PLUGINS_CMD + (_pluginsCommands.size() - 1);
					_pluginInfos[i]->_funcItems[j]._cmdID = cmdID;

					::InsertMenu(_pluginInfos[i]->_pluginMenu, j, MF_BYPOSITION, cmdID, _pluginInfos[i]->_funcItems[j]._itemName);
					if (_pluginInfos[i]->_funcItems[j]._init2Check)
						::CheckMenuItem(_hPluginsMenu, cmdID, MF_BYCOMMAND | MF_CHECKED);
				}
			}
		}
	};

	void notify(SCNotification *notification) {
		for (size_t i = 0 ; i < _pluginInfos.size() ; i++)
		{
			_pluginInfos[i]->_pBeNotified(notification);
		}
	};
private:
	NppData _nppData;
	
	HMENU _hPluginsMenu;
		
	vector<PluginInfo *> _pluginInfos;
	vector<PFUNCPLUGINCMD> _pluginsCommands;
	bool hasPlugins(){return (_pluginInfos.size()!= 0);};
};


#endif PLUGINSMANAGER_H
